<?php
/**
 *  Common HTTP protocol functions
 */
require_once("sessions.php");

class HTTP {
    const OK = 200;
    const CREATED = 201;
    const ERROR = 500;
    const LOCALHOST = "localhost";
    const SECONDARY_AUTH = "AuthToken";
    const EXPIRED_MESSAGE = "Your session has expired or is invalid.  Please log in again to continue.";
    const UNAUTHENTICATED_MESSAGE = "Request is not authenticated.";
    
    public function __construct($allowUnsupportedDevice = false, $allowExistingDevice = false) {
        $this->method = strtolower($_SERVER['REQUEST_METHOD']);
        $this->accept = 'json';
        $this->url = $_SERVER['REQUEST_URI'];
        $this->lastError = "";
        $headers = getallheaders();

        $this->data = null;
        switch ($this->method) {
            case 'get':
                $this->data = $_GET;
                break;
            case 'put':
            case 'post':
            case 'delete':
                $this->data = json_decode(file_get_contents('php://input'), true);
                break;
        }
        $this->auth = isset($headers['xg-token']) ? $headers['xg-token'] : "";
        $this->fromDevice = $this->isFromDevice($this->auth);
        if ($this->fromDevice) {
            $this->allowUnsupportedDevice = $allowUnsupportedDevice;
            $this->hasSupportContract = false;
        } else {
            $this->hasSupportContract = true;
        }
        // Allow local requests.
        $this->localHost = (isset($headers['Host']) && $headers['Host'] == HTTP::LOCALHOST) || (isset($headers['host']) && strpos($headers['host'], HTTP::LOCALHOST) === 0);
        if ($this->auth === "") {
            $this->secondaryAuth = isset($headers[HTTP::SECONDARY_AUTH]) ? $headers[HTTP::SECONDARY_AUTH] : "";
        } else {
            $this->secondaryAuth = "";
        }
    }

    public function getMethod() {
        return $this->method;
    }

    public function getData() {
        return $this->data;
    }

    public function isAuthenticated(&$message = "") {
        $valid = false;
        if ($this->localHost) {
            $valid = true;
        } else {
            if ($this->auth !== "") {
                if ($this->fromDevice) {
                    if (is_bool($this->allowUnsupportedDevice) && ($this->allowUnsupportedDevice || $this->authenticateDevice())) {
                        // valid if either we are allowing devices not under contract to be added, or it has a valid support contract.
                        $valid = true;
                    } else if ($this->allowUnsupportedDevice == Configuration::ALLOW_EXISTING_DEVICES) {
                        $valid = $this->deviceFound();
                        if ($valid === false) {
                            $message = "This device was not found or is not supported.";
                        }
                    }
                } else {
                    // Authenticate locally to the database
                    $session = new Sessions();
                    $valid = $session->isTokenValid($this->auth);
                    if ($valid === Sessions::EXPIRED_TOKEN){
                        // log this and delete sessions
                        global $Log;
                        $valid = false;
                        $Log->logInfo("Token is expired, deleting");
                        $session->deleteSession($this->auth);
                        if ($message !== null) {
                            $message = HTTP::EXPIRED_MESSAGE;
                        }
                    } else if ($valid === Database::CONNECT_ERROR) {
                        // log therror
                        global $Log;
                        $valid = false;
                        $Log->logError(Database::CONNECT_ERROR);
                        if ($message !== null) {
                            $message = Database::CONNECT_ERROR;
                        }
                    }
                }
            } else {
                // See if this is a request coming from a vendor Appliance.
                if ($this->secondaryAuth !== "") {
                    global $Log;
                    $Log->logInfo("Using vendor-specific authentication"); // . $this->secondaryAuth);
                    if (function_exists("bp_set_cookie")) {
                        $valid = bp_set_cookie($this->secondaryAuth);
                        if ($valid === false) {
                            $message = HTTP::EXPIRED_MESSAGE;
                        }
                    } else {
                        // Add other authentication methods here.
                        $Log->logInfo("No function for vendor-specific authentication");
                        $message = "No function provided for vendor-specific authentication.";
                        $valid = false;
                    }
                }
            }           
        }
        return $valid;
    }

    public function isSuccessful($result) {
        $success = true;
        if (is_array($result)) {
            if (isset($result['result']) && $result['result'] == false) {
                $success = false;
            }
        } else if ($result !== false) {
            $success = true;
        } else {
            $success = false;
        }
        return $success;
    }
    
    public function buildResult($result, $request) {
        if (is_array($result)) {
            if (isset($result['result']) && $result['result'] == false) {
                $message = isset($result['message']) ? $result['message'] : "Unknown error";
                $this->buildFailure(null, $message);
            } else {
                $this->buildSuccess($result, $request);
            }
        } else if ($result !== false) {
            $this->buildSuccess($result, $request);
        } else {
            $this->buildFailure(null, "Unknown server error");
        }
    }

    public function buildSuccess($result, $request = "get") {
        if ($result === true) {
            $result = array("result" => true);
        }
        if ($result !== false) {
            $json = json_encode($result);
            $httpCode = 'HTTP/1.1 ';
            $httpCode .= ($request === "post") ? HTTP::CREATED . ' Created' : HTTP::OK . ' OK';
            header($httpCode);
            header('Content-type: application/json');
            echo $json;
        }
    }

    public function buildTextResult($result, $request = "get") {
        if (is_string($result)) {
            $httpCode = 'HTTP/1.1 ';
            $httpCode .= ($request === "post") ? HTTP::CREATED . ' Created' : HTTP::OK . ' OK';
            header($httpCode);
            header('Content-type: text/plain');
            echo $result;
        } else {
            $this->buildFailure($result, "string result expected, not found");
        }
    }
    
    public function buildFailure($result, $message) {
        $httpCode = 'HTTP/1.1 ' . HTTP::ERROR . ' Internal Server Error';
        header($httpCode);
        $json = json_encode(array('message' => $message));
        header('Content-type: application/json');
        echo $json;
    }
    
    private function authenticateDevice() {
        $cmd = "/usr/local/mgportal/bin/vendor/lookup_asset asset %s check";
        //$password = Commands::getPortalPassword();
        //$format = "echo " . $password . " | sudo -u mgportal -S " . $cmd;
        $format = "sudo -u mgportal " . $cmd;
        $cmd = sprintf($format, $this->getAssetFromAuth($this->auth));
        exec($cmd, $outputArray, $return_value);
        $authentic = false;
        // If the asset is found, with DA, returns 0 - success);
        // otherwise, failure.  At times, result is non-zero but good.  Check for this.
        global $Log;
        $Log->logInfo(implode("", $outputArray));
        if ($return_value !== 0) {
            $this->lastError = implode(" ", $outputArray);
            // workaround - see if really valid!
            if (strstr($this->lastError, " is valid through ")) {
                $Log->logError("device check return value bad, but asset is valid");
                $authentic = true;
                $this->lastError = "";
            }
        } else {
            $authentic = true;
        }
        return $authentic;
    }

    private function deviceFound() {
        $found = false;
        $asset = $this->getAssetFromAuth($this->auth);
        $query = "SELECT id FROM mgportal.devices WHERE unique_identifier='" . $asset . "'";

        $db = new Database();
        $conn = $db->connect();
        $deviceResult = Database::db_query($conn, null, $query);
        if ($deviceResult !== false && Database::db_num_rows(null, $deviceResult) === 1){
            $found = true;
        }
        $db->closeConnection();
        return $found;
    }
    
    public function getError() {
        return $this->lastError;
    }

    public function getAuthToken() {
        return $this->auth;
    }
    
    private function isFromDevice($auth) {
        $fromDevice = false;
        $token = base64_decode($auth);
        $a = explode(":", $token);
        if (count($a) > 2) {
            $fromDevice = ($a[1] == 'xg-device');
        }
        return $fromDevice;        
    }
    
    private function getAssetFromAuth($auth) {
        $token = base64_decode($auth);
        $asset = "";
        $a = explode(":", $token);
        if (count($a) > 2) {
            $asset = $a[2];
        }
        return $asset;
    }
    
    public function getDeviceSupportStatus() {
        return $this->hasSupportContract;
    }
};

?>